var margin = {
        top: 20,
        right: 120,
        bottom: 20,
        left: 120
    },
    width = 3000 - margin.right - margin.left,
    height = window.innerHeight - margin.top - margin.bottom;

var i = 0,
    duration = 750,
    root;

var tree = d3.layout.tree()
    .size([height, width]);

var diagonal = d3.svg.diagonal()
    .projection(function(d) { return [d.y, d.x]; });

var svg = d3.select("body").append("svg")
    .attr("width", width)
    .attr("height", height + margin.top + margin.bottom)
    .append("g")
    .attr("transform", "translate(" + margin.left + "," + margin.top + ")");

d3.json("sized_ontology.json", function(error, nodes) {
  root = nodes;
  root.x0 = height / 2;
  root.y0 = 0;

  function collapse(d) {
    if (d.children) {
      if (!d.children[0].children) {
          d._children = d.children;
          d._children.forEach(collapse);
          d.children = null;
      } else {
          d.children.forEach(collapse);
      }
    }
  }

  root.children.forEach(collapse);

  update(root);
});

function resize() {
  root.x0 = height / 2;
  root.y0 = 0;
  update(root);
}

function zoom(size) {
    height = size;
    tree.size([height, width]);
    svg.attr("width", width).attr("height", height + margin.top + margin.bottom)
    document.querySelector('svg').attributes.height.value = height + margin.top + margin.bottom
    resize()
}


document.querySelector('.zoom button.plus').addEventListener('click', function(event) {
    zoom(height*1.5)
})
document.querySelector('.zoom button.minus').addEventListener('click', function(event) {
    zoom(height/1.5)
})


function get_color(d) {
  if (d.children || d._children) {
      return '#fff'
  }
  return "lightsteelblue";
}
function update(source) {

  // Compute the new tree layout.
  var nodes = tree.nodes(root).reverse(),
      links = tree.links(nodes);

  // Normalize for fixed-depth.
  nodes.forEach(function(d) { d.y = d.depth * 180; });

  // Update the nodes…
  var node = svg.selectAll("g.node")
      .data(nodes, function(d) {
          return d.id || (d.id = ++i);
      });

  // Enter any new nodes at the parent's previous position.
  var nodeEnter = node.enter().append("g")
      .attr("class", "node")
      .attr("transform", function(d) {
          return "translate(" + source.y0 + "," + source.x0 + ")";
      })
      .on("click", click);

  nodeEnter.append("circle")
      .attr("r", 1e-6)
      .style("fill", get_color)

  nodeEnter.append("text")
      .attr("x", function(d) {
          return d.children || d._children ? -10 : 10;
      })
      .attr("dy", ".35em")
      .attr("text-anchor", function(d) {
          return d.children || d._children ? "end" : "start";
      })
      .text(function(d) {
          if (d.children || d._children) {
              //return d.name;
              return d.name + '(' + (d.children || d._children).length +')';
          }
          return d.name + ' (' + d.size + ')';
      })
      .style("fill-opacity", 1e-6);


  // Transition nodes to their new position.
  var nodeUpdate = node.transition()
      .duration(duration)
      .attr("transform", function(d) {
          return "translate(" + d.y + "," + d.x + ")";
      });

  nodeUpdate.select("circle")
      .attr("r", 4.5)
      .style("fill", get_color)

  nodeUpdate.select("text")
      .style("fill-opacity", 1);

  // Transition exiting nodes to the parent's new position.
  var nodeExit = node.exit().transition()
      .duration(duration)
      .attr("transform", function(d) {
          return "translate(" + source.y + "," + source.x + ")";
      })
      .remove();

  nodeExit.select("circle")
      .attr("r", 1e-6);

  nodeExit.select("text")
      .style("fill-opacity", 1e-6);

  // Update the links…
  var link = svg.selectAll("path.link")
      .data(links, function(d) {
          return d.target.id;
      });

  // Enter any new links at the parent's previous position.
  link.enter().insert("path", "g")
      .attr("class", "link")
      .attr("d", function(d) {
        var o = {x: source.x0, y: source.y0};
        return diagonal({source: o, target: o});
      });

  // Transition links to their new position.
  link.transition()
      .duration(duration)
      .attr("d", diagonal);

  // Transition exiting nodes to the parent's new position.
  link.exit().transition()
      .duration(duration)
      .attr("d", function(d) {
        var o = {x: source.x, y: source.y};
        return diagonal({source: o, target: o});
      })
      .remove();

  // Stash the old positions for transition.
  nodes.forEach(function(d) {
    d.x0 = d.x;
    d.y0 = d.y;
  });
}

// Toggle children on click.
function click(d) {
  if (d.children) {
    d._children = d.children;
    d.children = null;
  } else {
    d.children = d._children;
    d._children = null;
  }
  if (!d.children && ! d._children) {
    var name = d.parent.name + ': ' + d.name
    var url = '/clip/created/keywords=' + name
    if (d.parent.name == 'other') {
      url = '/clip/created/keywords==' + d.name
    }
    window.open(url, '_blank');
  } else {
    update(d);
  }
}
